@@ -50,10 +50,24 @@ static int init () {
 }
 static int __init = init();
 
-char* encode_uri_component (const char* src, size_t srclen, char* dest, size_t* destlen, const char* unsafe) {
+void unsafe_generate (char* unsafe, int flags, const char* chars) {
+    if (flags & UNSAFE_DIGIT)      unsafe_generate(unsafe, 0, "0123456789");
+    if (flags & UNSAFE_ALPHA)      unsafe_generate(unsafe, 0, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
+    if (flags & UNSAFE_SUBDELIMS)  unsafe_generate(unsafe, 0, "!$&'()*+,;=");
+    if (flags & UNSAFE_GENDELIMS)  unsafe_generate(unsafe, 0, ":/?#[]@");
+    if (flags & UNSAFE_RESERVED)   unsafe_generate(unsafe, UNSAFE_SUBDELIMS | UNSAFE_GENDELIMS);
+    if (flags & UNSAFE_UNRESERVED) unsafe_generate(unsafe, UNSAFE_ALPHA | UNSAFE_DIGIT, "-._~");
+    if (flags & UNSAFE_PCHAR)      unsafe_generate(unsafe, UNSAFE_UNRESERVED | UNSAFE_SUBDELIMS, ":@");
+    if (chars) while (char c = *chars++) unsafe[(unsigned char) c] = c;
+}
+
+size_t encode_uri_component (const std::string_view src, char* dest, const char* unsafe) {
+    const char* str = src.data();
+    const char*const end = str + src.length();
     char* buf = dest;
-    for (int i = 0; i < srclen; ++i) {
-        uchar uc = src[i];
+
+    while (str != end) {
+        uchar uc = *str++;
         if (likely(unsafe[uc] != 0)) *buf++ = unsafe[uc];
         else {
             *buf++ = '%';
@@ -62,25 +76,24 @@ char* encode_uri_component (const char* src, size_t srclen, char* dest, size_t*
         }
     }
 
-    *buf = 0;
-    *destlen = buf - dest;
-    return dest;
+    return buf - dest;
 }
 
-char* decode_uri_component (const char* src, size_t srclen, char* dest, size_t* destlen) {
+size_t decode_uri_component (const std::string_view src, char* dest) {
+    const char* str = src.data();
+    const char*const end = str + src.length();
     char* buf = dest;
-    for (int i = 0; i < srclen; ++i) {
-        char res = _restore[(uchar)src[i]];
+
+    while (str != end) {
+        char res = _restore[(uchar)*str++];
         if (likely(res != 0)) *buf++ = res;
-        else if (i < srclen-2) {
-            *buf++ = _backward[(uchar)src[i+1]][0] | _backward[(uchar)src[i+2]][1];
-            i += 2;
+        else if (str < end-1) {
+            *buf++ = _backward[(uchar)str[0]][0] | _backward[(uchar)str[1]][1];
+            str += 2;
         }
     }
 
-    *buf = 0;
-    *destlen = buf - dest;
-    return dest;
+    return buf - dest;
 }
 
 }}
